/**
	\file
	\brief Functions for reading and writing catalogue files

	Catalogue files contain information relative position
	of stars and position of variable, comparison and check
	stars. It is designed to simplify the final phase of the 
	reduction process. The application loads the selection
	from a catalogue file and user only checks and confirms it.

	The catalogue file is a XML document, see the Reference
	manual for its document type definition.

	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_catfile.h,v 1.2 2010/04/24 19:10:09 dmotl Exp $
*/

#ifndef _CMPACK_CATFILE_H_INCLUDED
#define _CMPACK_CATFILE_H_INCLUDED

#include "cmpack_phtfile.h"

/*****************    Data types    ***************************/

/** 
	\brief Selection types 
*/
typedef enum _CmpackSelectionType
{
	CMPACK_SELECT_NONE,		/**< Ordinary star */
	CMPACK_SELECT_VAR,		/**< Variable star */
	CMPACK_SELECT_COMP,		/**< Comparison star */
	CMPACK_SELECT_CHECK		/**< Check star */
} CmpackSelectionType;

/**	
	\brief Catalogue file context 
	\details This private structure holds the content of the catalogue file.
*/
typedef struct _CmpackCatFile CmpackCatFile;

/**
	\brief Star definition in catalogue file
	\details This structure is used to access a star definitions
	in a catalogue files.
*/
typedef struct _CmpackCatStar CmpackCatStar;

/**
	\brief Selection in catalogue file
	\details This structure is used to access a selection of stars
	in a catalogue files.
*/
typedef struct _CmpackCatSelection CmpackCatSelection;

/********************   Functions   **************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Create a new catalogue file context
	\return pointer to a new instance
*/
CmpackCatFile *cmpack_cat_init(void);
		
/**
	\brief Load catalogue file from file
	\param[out]	file		new file context
	\param[in] 	filename	path + file name
	\param[in] 	flags 		flags (see CMPACK_LOAD_xxx constants)
	\return zero on success, error code on failure
*/
int cmpack_cat_load(CmpackCatFile **file, const char *filename, CmpackLoadMode flags);

/**
	\brief Save changed, close file and free allocated memory.
	\details If the file was created by means of cmpack_cat_init() function,
	no changes are saved.
	\param[in]	file 		file context
	\param[in] 	filename	path + file name
	\return zero on success, error code on failure
*/
int cmpack_cat_save(CmpackCatFile *file, const char *filename);

/**
	\brief Clear all data
	\param[in]	file		file context
*/
void cmpack_cat_clear(CmpackCatFile *file);

/** 
	\brief Make a copy of a catalogue file
	\details Makes a copy of source file to the destination file.
	All content of the destination file will be deleted.
	\param[in] dstfile		destination file context
	\param[in] srcfile		source file context
*/
void cmpack_cat_copy(CmpackCatFile *dstfile, CmpackCatFile *srcfile);

/** 
	\brief Make catalogue file from a photometry file
	\details Converts a photometry file to a catalogue file.
	All content of the destination file will be deleted.
	\param[in] dstfile		destination file context
	\param[in] srcfile		source file context
	\param[in] aperture		aperture id
*/
int cmpack_cat_make(CmpackCatFile *dstfile, CmpackPhtFile *srcfile, int aperture);

/** 
	\brief Test if given file seems to be a catalogue file
	\return nonzero if the file is a photometry file, zero otherwise
*/
int cmpack_cat_test(const char *filename);

/** 
	\brief Test if the content of the buffer might be a catalogue file
	\details The function checks the data in the buffer and returns nonzero,
	when the buffer contain first 'buflen' bytes from a catalogue file.
	This function is used for file format autodetection.
	\return nonzero if the file is a photometry file, zero otherwise
*/
int cmpack_cat_test_buffer(const char *buffer, int buflen, int filesize);

/************************** File header *******************************************/

/**
	\brief Get frame size in pixels
	\param[in] file 			file context
	\param[out] width			frame width in pixels (0 = not available)
	\param[out] height			frame height in pixels (0 = not available)
	\return nonzero on success, zero on failure
*/
int cmpack_cat_get_size(CmpackCatFile *file, int *width, int *height);

/**
	\brief Get julian date of observation
	\param[in] fc				file context
	\return Julian date or zero on failure
*/
double cmpack_cat_get_jd(CmpackCatFile *fc);

/**
	\brief Get exposure duration
	\param[in] fc				file context
	\return Exposure duration in seconds or zero on failure
*/
double cmpack_cat_get_exptime(CmpackCatFile *fc);

/**
	\brief Get optical filter name
	\details The function returns pointer to the internal buffer. 
	You must not free or modify it.
	\param[in] fc				file context
	\return Filter name, null terminated string
*/
const char *cmpack_cat_get_filter(CmpackCatFile *fc);

/**
	\brief Get object's designation
	\details The function returns pointer to the internal buffer. 
	You must not free or modify it.
	\param[in] fc				file context
	\return Object's designation, null terminated string
*/
const char *cmpack_cat_get_object(CmpackCatFile *fc);

/**
	\brief Get object's right ascension (R.A.)
	\details The function returns pointer to the internal buffer. 
	You must not free or modify it.
	\param[in] fc				file context
	\return Object's R.A., null terminated string
*/
const char *cmpack_cat_get_objra(CmpackCatFile *fc);

/**
	\brief Get object's declination (Dec.)
	\details The function returns pointer to the internal buffer. 
	You must not free or modify it.
	\param[in] fc				file context
	\return Object's Dec., null terminated string
*/
const char *cmpack_cat_get_objdec(CmpackCatFile *fc);

/**
	\brief Set parameter value in file header (string)
	\details If the parameter does not exist, it makes new entry in
	the file header. If the parameter exists, its content is changed. 
	Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\param[in] com			comment (parameter description)
*/
void cmpack_cat_pkys(CmpackCatFile *file, const char *key, const char *val, const char *com);

/**
	\brief Set parameter value in file header (integer)
	\details If the parameter does not exist, it makes new entry in
	the file header. If the parameter exists, its content is changed. 
	Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\param[in] com			comment (parameter description)
	\return zero on success, error code on failure
*/
void cmpack_cat_pkyi(CmpackCatFile *file, const char *key, int val, const char *com);

/**
	\brief Set parameter value in file header (double)
	\details If the parameter does not exist, it makes new entry in
	the file header. If the parameter exists, its content is changed. 
	Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\param[in] prec			number of decimal places
	\param[in] com			comment (parameter description)
	\return zero on success, error code on failure
*/
void cmpack_cat_pkyd(CmpackCatFile *file, const char *key, double val, int prec, const char *com);
	
/**
	\brief Get value of parameter in file header (string)
	\param[in] file			file context
	\param[in] key			parameter name
	\return pointer to internal buffer or NULL
*/
const char *cmpack_cat_gkys(CmpackCatFile *file, const char *key);

/**
	\brief Get value of parameter in file header (integer number)
	\details Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] defval		default value
	\return parameter value or default value
*/
int cmpack_cat_gkyi(CmpackCatFile *file, const char *key, int defval);

/**
	\brief Get value of parameter in file header (real number)
	\details Keys are case sensitive. You can set the output parameter to NULL
	if you want to test the value only.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] defval		default value
	\return parameter value or default value
*/
double cmpack_cat_gkyd(CmpackCatFile *file, const char *key, double defval);

/**
	\brief Get comment text of parameter in file header
	\details Keys are case sensitive. The function returns pointer to 
	the internal buffer. You must not free or modify it.
	\param[in] file			file context
	\param[in] key			parameter name
	\return comment text, null terminated string
*/
const char *cmpack_cat_gkyc(CmpackCatFile *file, const char *key);

/**
	\brief Get number of parameters in file header
	\param[in] file			file context
	\return number of parameter or zero if the file
	is not valid context or if the header is empty.
*/
int cmpack_cat_nkey(CmpackCatFile *file);

/**
	\brief Get the parameter from file header by its index
	\details Index of the first parameter is zero. The output
	parameters receive the pointers to newly allocated memory buffers. 
	The caller is responsible to free it by calling of cmpack_free() 
	function. You can set the value of output parameter to NULL if you
	do not need to read it.
	\param[in] file			file context
	\param[in] index		parameter index (starting by zero)
	\param[out] key			parameter name (can be NULL)
	\param[out] val			parameter value (can be NULL)
	\param[out] com			comment (can be NULL)
	\return zero on success, error code on failure
*/
int cmpack_cat_gkyn(CmpackCatFile *file, int index, const char **key, const char **val, const char **com); 

/**
	\brief Delete parameter from file header
	\details Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
*/
void cmpack_cat_dkey(CmpackCatFile *file, const char *key);

/*************************** Table of stars **********************************/

/**
	\brief Gets number of stars defined in the file
	\param[in] file			file context
	\return number of selected stars or zero on failure
*/
int cmpack_cat_nstar(CmpackCatFile *file);

/**
	\brief Add a star to a catalogue file
	\param[in] file			file context
	\param[in] star_id		star identifier
	\return new reference to a star
*/
CmpackCatStar *cmpack_cat_add_star(CmpackCatFile *file, int star_id);

/**
	\brief Get star by its index
	\param[in] file			file context
	\param[in] index		index
*/	
CmpackCatStar *cmpack_cat_get_star(CmpackCatFile *file, int index);

/**
	\brief Find star by identifier
	\param[in] file			file context
	\param[in] star_id		star identifier
*/	
CmpackCatStar *cmpack_cat_find_star(CmpackCatFile *file, int star_id);

/**
	\brief Get star indentifier
	\param[in] star			star context
	\return identifier of a star or negative value on failure
*/
int cmpack_cat_star_get_id(CmpackCatStar *star);

/**
	\brief Set position of a star
	\param[in] star			star context
	\param[in] x			position in x axis in pixels
	\param[in] y			position in y axis in pixels
*/
void cmpack_cat_star_set_center(CmpackCatStar *star, double x, double y);

/**
	\brief Get position of a star
	\param[in] star			star context
	\param[out] x			position in x axis in pixels
	\param[out] y			position in y axis in pixels
	\return zero on success or error code on failure
*/
int cmpack_cat_star_get_center(CmpackCatStar *star, double *x, double *y);

/**
	\brief Set magnitude of a star
	\param[in] star			star context
	\param[in] mag			magnitude
*/
void cmpack_cat_star_set_magnitude(CmpackCatStar *star, double mag);

/**
	\brief Get magnitude of a star
	\param[in] star			star context
	\param[out] mag			magnitude
	\return nonzero on success or zero on failure
*/
int cmpack_cat_star_get_magnitude(CmpackCatStar *star, double *mag);

/*************************** Selection **********************************/

/**
	\brief Set selection type of all stars to CMPACK_SELECT_NONE
	\param[in] file			file context
*/
void cmpack_cat_clear_selection(CmpackCatFile *file);

/**
	\brief Set selection type of a single star
	\param[in] file			file context
	\param[in] star_id		star identifier
	\param[in] type			selection type
*/
void cmpack_cat_set_star_type(CmpackCatFile *file, int star_id, CmpackSelectionType type);

/**
	\brief Get selection type of a single star
	\param[in] file			file context
	\param[in] star_id		star identifier
	\return selection type or CMPACK_SELECT_NONE if the star does not exist.
*/
CmpackSelectionType cmpack_cat_get_star_type(CmpackCatFile *file, int star_id);

/**
	\brief Get selection context from a catalogue file
	\param[in] file			file context
	\param[in] type			selection type
	\returns new reference to the selection context
*/
CmpackCatSelection *cmpack_cat_get_selection(CmpackCatFile *file, CmpackSelectionType type);

/**
	\brief Clear the selection context
	\param[in] sel			selection context
*/
void cmpack_cat_selection_clear(CmpackCatSelection *sel);

/**
	\brief Append a list of stars 
	\param[in] sel			selection context
	\param[in] list			star identifiers
	\param[in] count		number of stars
*/
void cmpack_cat_selection_append(CmpackCatSelection *sel, const int *list, int count);

/**
	\brief Set selection
	\param[in] sel			selection context
	\param[in] list			star identifiers
	\param[in] count		number of stars
*/
void cmpack_cat_selection_set(CmpackCatSelection *sel, const int *list, int count);

/**
	\brief Get number of stars in a selection
	\param[in] sel			selection context
	\return number of items or zero on failure
*/
int cmpack_cat_selection_nitems(CmpackCatSelection *sel);

/**
	\brief Get list of stars
	\param[in] sel			selection context
	\param[in] list			buffer which receives star identifiers
	\param[in] count		buffer size 
	\return number of items stored to buffer or negative value on failure
*/
int cmpack_cat_selection_get(CmpackCatSelection *sel, int *list, int count);

/**
	\brief Get a star by index
	\param[in] sel			selection context
	\param[in] index		zero base index
	\return star identifier or negative value on failure
*/
int cmpack_cat_selection_item(CmpackCatSelection *sel, int index);

#ifdef __cplusplus
}
#endif

#endif
