/**
	\file
	\brief Console related functions

	C-Munipack functions use a console as an output device for
	output and debug messages. By default, all messages are thrown away.
	If you want to print them to standard error stream or process them
	in a user defined callback, make a console and attach it to an object
	using cmpack_*_set_console function.

	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_console.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/

#ifndef _CMPACK_CONSOLE_H_INCLUDED
#define _CMPACK_CONSOLE_H_INCLUDED

/********************   Public constants   ********************************/

/** \brief Verbosity level */
typedef enum _CmpackOutputLevel
{
	CMPACK_LEVEL_QUIET,			/**< Be quiet, suppress all messages */
	CMPACK_LEVEL_NORMAL,		/**< Print messages, suppress debug output (default) */
	CMPACK_LEVEL_DEBUG			/**< Be verbose, print all messages including debug ones */
} CmpackOutputLevel;

/********************   Public data types   ********************************/

/**
	\brief Callback procedure
	\details Prototype for custom callback procedures. The procedure
	receives the pointer to message string. It points to internal memory
	buffer, the callee must not free or modify it. The second parameter 
	contains callback registration data.
	\param[in] text			text to be printed to output device
	\param[in] user_data	callback registration data
*/
typedef void CmpackCallbackType(const char *text, void *user_data);

/**	
	\brief Console context 
	\details This private structure holds the console parameters
*/
typedef struct _CmpackConsole CmpackConsole;

/******************   Public functions   ***************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Create a new console context that prints all output to stderr stream
	\return pointer to context of zero if failed
*/
CmpackConsole *cmpack_con_init(void);

/**
	\brief Create a new console context that uses a user-defined function 
	to print messages.
	\return pointer to context of zero if failed
*/
CmpackConsole *cmpack_con_init_cb(CmpackCallbackType *cbproc, void *cbdata);

/**
	\brief Set the debug level
	\param[in] ctx			any context
	\param[in] level		verbosity level (see CMPACK_LEVEL_xxx constants)
*/
void cmpack_con_set_level(CmpackConsole *ctx, CmpackOutputLevel level);

#ifdef __cplusplus
}
#endif

#endif
