/**
	\file
	\brief Functions for the dark-frame correction

	Set of functions defined in this module allows user to 
	apply dark-frame correction to CCD frames.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_dark.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/

#ifndef _CMPACK_DARK_H_INCLUDED
#define _CMPACK_DARK_H_INCLUDED

#include "cmpack_common.h"
#include "cmpack_console.h"
#include "cmpack_ccdfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Dark-frame correction context
	\details This private data structure holds the parameter for dark-frame
	correction as well as the dark frame itself.
*/
typedef struct _CmpackDarkCorr CmpackDarkCorr;

/********************   Public functions   *************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new dark-frame correction context
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to context or zero on failure
*/
CmpackDarkCorr *cmpack_dark_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			air-mass computation context
	\param[in] con			console context
*/
void cmpack_dark_set_console(CmpackDarkCorr *ctx, CmpackConsole *con);

/**
	\brief Enable or disable dark-frame scaling
	\param[in] ctx			air-mass computation context
	\param[in] scaling		0 = disable, 1 = enable feature
*/
void cmpack_dark_set_scaling(CmpackDarkCorr *ctx, int scaling);

/**
	\brief Get status of dark-frame scaling
	\param[in] ctx			air-mass computation context
	\return zero = disabled, nonzero = enabled
*/
int cmpack_dark_get_scaling(CmpackDarkCorr *ctx);

/**
	\brief Set image border size
	\details If you set the border to nonzero size, the conversion function
	will set the pixels which belongs to the border area to zero. You can use
	this feature to clear an unusable part of a frame.
	\param[in] file			conversion context
	\param[in] border		border size in pixels
*/
void cmpack_dark_set_border(CmpackDarkCorr *file, const CmpackBorder *border);

/**
	\brief Get image flip flags
	\param[in] file			conversion context
	\param[out] border		border size in pixels
*/
void cmpack_dark_get_border(CmpackDarkCorr *file, CmpackBorder *border);

/**
	\brief Load dark-frame from a file
	\details The function reads the correction frame from the given frame 
	context. The internal copy of the image data is made, no reference to 
	the frame is held, so you can free it when this function has returned.
	\param[in] ctx			dark-frame correction context
	\param[in] darkfile		dark frame context
	\return zero on success or error code on failure
*/
int cmpack_dark_rdark(CmpackDarkCorr *ctx, CmpackCcdFile *darkfile);

/**
	\brief Execute dark-frame correction on a file
	\details The function reads image data from the context, performs
	the dark correction and stores the output image to the same context.
	\param[in] ctx			dark-frame correction context
	\param[in,out] file		frame context
	\return zero on success or error code on failure
*/
int cmpack_dark(CmpackDarkCorr *ctx, CmpackCcdFile *file);

/**
	\brief Execute dark-frame correction on a file
	\details The function reads image data from the infile context, performs
	the dark correction and stores the output image to the outfile context.
	\param[in] ctx			dark-frame correction context
	\param[in] infile		input frame context
	\param[in] outfile		output frame context
	\return zero on success or error code on failure
*/
int cmpack_dark_ex(CmpackDarkCorr *ctx, CmpackCcdFile *infile, CmpackCcdFile *outfile);

#ifdef __cplusplus
}
#endif

#endif
