/**
	\file
	\brief Functions for the merging (combining) CCD frames

	Set of functions defined in this module allows user to 
	merge (combine) a set of CCD frames.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_kombine.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_KOMBINE_H_INCLUDED
#define _CMPACK_KOMBINE_H_INCLUDED

#include "cmpack_common.h"
#include "cmpack_console.h"
#include "cmpack_ccdfile.h"
#include "cmpack_phtfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Output frame context for kombine tool 
	\details This private data structure holds the accumulated
	data for single output frame.
*/
typedef struct _CmpackKombine CmpackKombine;

/********************   Public functions   *************************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new kombine context
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to new kombine context or zero on failure
*/
CmpackKombine *cmpack_kombine_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			kombine context
	\param[in] con			console context
*/
void cmpack_kombine_set_console(CmpackKombine *ctx, CmpackConsole *con);

/**
	\brief Set output data format
	\param[in] ctx			output file context
	\param[in] bitpix		output data format
	\return zero on success or error code on failure
*/
void cmpack_kombine_set_bitpix(CmpackKombine *ctx, CmpackBitpix bitpix);

/**
	\brief Get output data format
	\param[in] ctx			output file context
	\return output data format
*/
CmpackBitpix cmpack_kombine_get_bitpix(CmpackKombine *ctx);

/**
	\brief Set image border size
	\details If you set the border to nonzero size, the conversion function
	will set the pixels which belongs to the border area to zero. You can use
	this feature to clear an unusable part of a frame.
	\param[in] ctx			conversion context
	\param[in] border		border size in pixels
*/
void cmpack_kombine_set_border(CmpackKombine *ctx, const CmpackBorder *border);

/**
	\brief Get image flip flags
	\param[in] ctx			conversion context
	\param[out] border		border size in pixels
*/
void cmpack_kombine_get_border(CmpackKombine *ctx, CmpackBorder *border);

/**
	\brief Open output file
	\details Opens new combined frame. Close the file by calling the 
	cmpack_kombine_close() function.
	\param[in] ctx			kombine context
	\param[in] outfile		output file context
	\return zero on success or error code on failure.
*/
int cmpack_kombine_open(CmpackKombine *ctx, CmpackCcdFile *outfile);

/**
	\brief Add a frame to the accumulation buffer
	\details The function reads the frame from specified CCD-frame 
	file. It reads also the position offset from specified photometry
	file. Then shifts the frame by the offset and adds it to the accumulation
	buffer. 
	\param[in] ctx			kombine context
	\param[in] ccdfile		CCD frame file
	\param[in] phtfile		photometry file
	\return zero on success or error code on failure.
*/
int cmpack_kombine_read(CmpackKombine *ctx, CmpackCcdFile *ccdfile, CmpackPhtFile *phtfile);

/**
	\brief Make combined CCD frame and writes it to the file
	\details The function computes the output CCD data, writes it
	to the file, closes the output frame.
	\param[in] ctx			kombine context
	\return zero on success or error code on failure.
*/
int cmpack_kombine_close(CmpackKombine *ctx);
	
#ifdef __cplusplus
}
#endif

#endif
