/**
	\file
	\brief Functions for converting CCD frames from vendor
	specific format to the FITS format.

	Set of functions defined in this module allows user to 
	convert the CCD frames to FITS format.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_konve.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_KONVE_H_INCLUDED
#define _CMPACK_KONVE_H_INCLUDED

#include "cmpack_common.h"

/********************   Private data structures   ********************************/

/**
	\brief Conversion context
	\details This private data structure holds the configuration parameters for 
	CCD-frame conversion.
*/
typedef struct _CmpackKonv CmpackKonv;

/********************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new conversion context
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to context or zero on failure
*/
CmpackKonv *cmpack_konv_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			conversion context
	\param[in] con			console context
*/
void cmpack_konv_set_console(CmpackKonv *ctx, CmpackConsole *con);

/**
	\brief Set output data format
	\param[in] file			conversion context
	\param[in] bitpix		output data format
*/
void cmpack_konv_set_bitpix(CmpackKonv *file, CmpackBitpix bitpix);

/**
	\brief Get output data format
	\param[in] file			conversion context
	\return output data format
*/
CmpackBitpix cmpack_konv_get_bitpix(CmpackKonv *file);

/**
	\brief Set image flip flags
	\param[in] file			conversion context
	\param[in] bitmask		bitmask of image trasformation flags
*/
void cmpack_konv_set_trafo(CmpackKonv *file, int bitmask);

/**
	\brief Get image flip flags
	\param[in] file			conversion context
	\return bitmask of image transformation flags
*/
int cmpack_konv_get_trafo(CmpackKonv *file);

/**
	\brief Set image border size
	\details If you set the border to nonzero size, the conversion function
	will set the pixels which belongs to the border area to zero. You can use
	this feature to clear an unusable part of a frame.
	\param[in] file			conversion context
	\param[in] border		border size in pixels
*/
void cmpack_konv_set_border(CmpackKonv *file, const CmpackBorder *border);

/**
	\brief Get image flip flags
	\param[in] file			conversion context
	\param[out] border		border size in pixels
*/
void cmpack_konv_get_border(CmpackKonv *file, CmpackBorder *border);

/**
	\brief Convert a specified file to FITS format
	\details This function detects the format of specified source file, 
	read its header and image data and writes it to the output file.
	\param[in] lc			conversion context
	\param[in] infile		source file context
	\param[in] outfile		output file context
	\return zero on success or error code on failure
*/
int cmpack_konv(CmpackKonv *lc, CmpackCcdFile *infile, CmpackCcdFile *outfile);

#ifdef __cplusplus
}
#endif

#endif
