/**
	\file
	\brief Functions for making output data files.

	Set of functions defined in this module allows user to 
	read a set of information from photometry files and make
	output file (a light curve, a track list, ...)
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_list.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_LIST_H_INCLUDED
#define _CMPACK_LIST_H_INCLUDED

#include "cmpack_console.h"
#include "cmpack_phtfile.h"
#include "cmpack_table.h"

/********************   Private data structures   ********************************/

/**
	\brief Listing process context
	\details This private data structure holds the temporary data, which are
	used during listing process.
*/
typedef struct _CmpackLister CmpackLister;

/**********************   Public constants   ***********************************/

/**
	\brief Output format identifiers 
*/
typedef enum _CmpackListFormat
{
	CMPACK_LIST_DIFFMAG,		/**< Differential magnitudes (default) */
	CMPACK_LIST_INSTMAG,		/**< Instrumental magnitudes */
	CMPACK_LIST_TRACKLST,		/**< Track list */
	CMPACK_LIST_STDDEV 			/**< Standard deviations */
} CmpackListFormat;

/********************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new listing context
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to context or zero on failure
*/
CmpackLister *cmpack_list_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			air-mass computation context
	\param[in] con			console context
*/
void cmpack_list_set_console(CmpackLister *ctx, CmpackConsole *con);

/**
	\brief Set aperture index
	\param[in] proc			listing context
	\param[in] index		aperture index (starts by 1)
*/
void cmpack_list_set_aperture(CmpackLister *proc, int index);

/**
	\brief Get aperture index
	\param[in] proc			listing context
	\return aperture index (starts by 1)
*/
int cmpack_list_get_aperture(CmpackLister *proc);

/**
	\brief Set list of variable stars
	\param[in] proc			listing context
	\param[in] items		array of star identifiers
	\param[in] nitems		number of items
*/
void cmpack_list_set_var(CmpackLister *proc, const int *items, int nitems);

/**
	\brief Set list of comparison stars
	\param[in] proc			listing context
	\param[in] items		array of star identifiers
	\param[in] nitems		number of items
*/
void cmpack_list_set_comp(CmpackLister *proc, const int *items, int nitems);

/**
	\brief Set list of check stars
	\param[in] proc			listing context
	\param[in] items		array of star identifiers
	\param[in] nitems		number of items
*/
void cmpack_list_set_check(CmpackLister *proc, const int *items, int nitems);

/**
	\brief Start listing to a table
	\details The function clears the content of the table, sets its
	columns. Use cmpack_list_close() to finish the operation. If the
	'table' parameter is NULL, it makes a new table instance. You
	can get the pointer to it by calling the cmpack_list_get_table
	function.
	\param[in] proc			listing process context
	\param[in] table		table context
	\param[in] format		output format
	\param[in] flags		flags
	\return zero on success or error code on failure
*/
int cmpack_list_open(CmpackLister *proc, CmpackTable *table, CmpackListFormat format, int flags);

/**
	\brief Read single frame from a photometry file and process it
	\param[in] proc			listing process context
	\param[in] file			matched photometry file context
	\param[in] frame		frame ordinal number
	\return zero on success or error code on failure
*/
int cmpack_list_read(CmpackLister *proc, CmpackPhtFile *file, int frame);

/**
	\brief Flush all data, close reading sequence.
	\details The function finishes up the computation of output data.
	\param[in] proc			listing process context
	\return zero on success or error code on failure
*/
int cmpack_list_close(CmpackLister *proc);

#ifdef __cplusplus
}
#endif

#endif
