/**
	\file
	\brief Functions for the making master-dark correction frame

	Set of functions defined in this module allows user to 
	make a master-dark frame from a set of CCD frames.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_mdark.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_MDARK_H_INCLUDED
#define _CMPACK_MDARK_H_INCLUDED

#include "cmpack_common.h"
#include "cmpack_console.h"
#include "cmpack_ccdfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Configuration context for the Master-dark tool
	\details This private data structure holds the configuration parameters
	for the master-dark tool
*/
typedef struct _CmpackMasterDark CmpackMasterDark;

/********************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new context for the Master-dark tool
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to new master-dark context or zero on failure
*/
CmpackMasterDark *cmpack_mdark_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			master-dark context
	\param[in] con			console context
*/
void cmpack_mdark_set_console(CmpackMasterDark *ctx, CmpackConsole *con);

/**
	\brief Set output data format
	\param[in] ctx			master-dark context
	\param[in] bitpix		output data format
	\return zero on success or error code on failure
*/
int cmpack_mdark_set_bitpix(CmpackMasterDark *ctx, CmpackBitpix bitpix);

/**
	\brief Get output data format
	\param[in] ctx			master-dark context
	\param[out] bitpix		output data format
	\return zero on success or error code on failure
*/
int cmpack_mdark_get_bitpix(CmpackMasterDark *ctx, CmpackBitpix *bitpix);

/**
	\brief Set output data format
	\param[in] ctx			master-dark context
	\param[in] scalable		1 = make scalable dark frame
	\return zero on success or error code on failure
*/
int cmpack_mdark_set_scalable(CmpackMasterDark *ctx, int scalable);

/**
	\brief Get output data format
	\param[in] ctx			master-dark context
	\param[out] scalable	1 = make scalable dark frame
	\return zero on success or error code on failure
*/
int cmpack_mdark_get_scalable(CmpackMasterDark *ctx, int *scalable);

/**
	\brief Set image border size
	\details If you set the border to nonzero size, the conversion function
	will set the pixels which belongs to the border area to zero. You can use
	this feature to clear an unusable part of a frame.
	\param[in] ctx			master-dark context
	\param[in] border		border size in pixels
*/
void cmpack_mdark_set_border(CmpackMasterDark *ctx, const CmpackBorder *border);

/**
	\brief Get image flip flags
	\param[in] ctx			master-dark context
	\param[out] border		border size in pixels
*/
void cmpack_mdark_get_border(CmpackMasterDark *ctx, CmpackBorder *border);

/**
	\brief Open output file
	\details Opens new master-dark frame.
	The caller is responsible to close the file
	by calling the cmpack_mdark_close() function.
	\param[in] ctx			master-dark context
	\param[in] outfile		output file context
	\return zero on success or error code on failure.
*/
int cmpack_mdark_open(CmpackMasterDark *ctx, CmpackCcdFile *outfile);

/**
	\brief Add a frame to the accumulation buffer
	\details The function reads the frame from specified CCD-frame 
	file into memory.
	\param[in] ctx			output file context
	\param[in] infile		input file context
	\return zero on success or error code on failure.
*/
int cmpack_mdark_read(CmpackMasterDark *ctx, CmpackCcdFile *infile);

/**
	\brief Make master-dark frame and save it to the file
	\details The function computes the output CCD data, writes it
	to the file, closes the output file and frees allocated data.
	\param[in] ctx			output file context
	\return zero on success or error code on failure.
*/
int cmpack_mdark_close(CmpackMasterDark *ctx);

#ifdef __cplusplus
}
#endif

#endif
