/**
	\file
	\brief Functions for the making mag-dev graph

	Set of functions defined in this module allows user to 
	make a mag-dev graph, that is used to find variable stars
	in a viewfield.
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_mfind.h,v 1.1 2010/04/22 19:44:18 dmotl Exp $
*/
#ifndef _CMPACK_MFIND_H_INCLUDED
#define _CMPACK_MFIND_H_INCLUDED

#include "cmpack_console.h"
#include "cmpack_table.h"
#include "cmpack_allfile.h"

/********************   Private data structures   ********************************/

/**
	\brief Munifind context
	\details This private data structure holds information needed to build
	the mag-dev graph.
*/
typedef struct _CmpackMuniFind CmpackMuniFind;

/**********************   Public constants   ***********************************/

#define CMPACK_MFIND_AUTO	(-1)

/********************   Public functions   ********************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Make new context for the Munifind tool
	\details The reference counter is set to one. The caller is responsible
	to call cmpack_unref() when it is no longer needed.
	\return pointer to context or zero on failure
*/
CmpackMuniFind *cmpack_mfind_init(void);

/**
	\brief Attach console to the context
	\details Increment console's reference counter. Only one console can
	be attached to a single context. If another console is attached, by
	calling this function dettaches it. Set console to NULL to dettach
	the current console.
	\param[in] ctx			munifind context
	\param[in] con			console context
*/
void cmpack_mfind_set_console(CmpackMuniFind *ctx, CmpackConsole *con);

/**
	\brief Get comparison star 
	\param[in] ctx			munifind context
	\return identifier of the comparison star or negative value on error
*/
int cmpack_mfind_get_comparison(CmpackMuniFind *ctx);

/**
	\brief Set clipping threshold
	\param[in] ctx			munifind context
	\param[in] threshold	threshold in %
*/
void cmpack_mfind_set_threshold(CmpackMuniFind *ctx, double threshold);

/**
	\brief Make mag-dev graph
	\details The result is stored to a given table
	\param[in] ctx			munifind context
	\param[in] file			readall file
	\param[out] table		table with results
	\param[in] comp_star	comparison star id or CMPACK_MFIND_AUTO
	\return zero on success or error code on failure.
*/
int cmpack_mfind(CmpackMuniFind *ctx, CmpackAllFile *file, CmpackTable *table,
				 int comp_star);

#ifdef __cplusplus
}
#endif

#endif
