/**
	\file
	\brief Functions for reading and writing photometry files.

	Photometry files contain photometry result for a single CCD frame.
	The file is valid standalone XML document. This format was introduced
	in the version 1.2
	
	\author David Motl <dmotl@volny.cz>
	
	\par Copying
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation, version 2.
	
	$Id: cmpack_phtfile.h,v 1.2 2010/04/24 19:10:09 dmotl Exp $
*/

#ifndef _CMPACK_PHFILE_H_INCLUDED
#define _CMPACK_PHFILE_H_INCLUDED

/*****************    Private data structures    ***************************/

/** 
	\brief Photometry file context
	\details This private structure holds the content of the read-all file.
*/
typedef struct _CmpackPhtFile CmpackPhtFile;

/**
	\brief Aperture definition in photometry file
	\details This structure is used to access an aperture definitions
	in photometry files.
*/
typedef struct _CmpackPhtAperture CmpackPhtAperture;

/**
	\brief Star definition in photometry file
	\details This structure is used to access a star definitions
	in photometry files.
*/
typedef struct _CmpackPhtStar CmpackPhtStar;

/**
	\brief Measurement stored in photometry file
	\details This structure is used to specify/read the brightness
	measurements from a photometry file.
*/
typedef struct _CmpackPhtMeasurement
{
	int mask;					/**< Bitmask of valid fields (combination of CMPACK_MASK_xxx values) */
	int code;					/**< Result of photometry */
	double magnitude;			/**< Instrumental mahnitude */
	double error;				/**< Error estimation of instrumental magnitude */
} CmpackPhtMeasurement;

/********************   Public functions   **************************/

#ifdef __cplusplus
extern "C" {
#endif

/**
	\brief Create new photometry file context
	\return pointer to a new instance
*/
CmpackPhtFile *cmpack_pht_init(void);

/**
	\brief Load a photometry file
	\param[out]	file		new file context
	\param[in] 	filename	path + file name
	\param[in]	flags		flags (see CMPACK_LOAD_xxx constants)
	\return zero on success, error code on failure
*/
int cmpack_pht_load(CmpackPhtFile **file, const char *filename, CmpackLoadMode flags);

/**
	\brief Save data from memory to a photometry file
	\param[in]	file		file context
	\param[in] 	filename	path + file name
	\return zero on success, error code on failure
*/
int cmpack_pht_save(CmpackPhtFile *file, const char *filename);

/**
	\brief Copy a file
	\details Makes a copy of source file to the destination file.
	All content of the destination file will be deleted.
	\param[in] dstfile		destination file context
	\param[in] srcfile		source file context
*/
void cmpack_pht_copy(CmpackPhtFile *dstfile, CmpackPhtFile *srcfile);

/**
	\brief Clear all data
	\param[in]	file		file context
*/
void cmpack_pht_clear(CmpackPhtFile *file);

/** 
	\brief Test if given file seems to be a photometry file
	\return nonzero if the file is a photometry file, zero otherwise
*/
int cmpack_pht_test(const char *filename);

/** 
	\brief Test if the content of the buffer might be a photometry file
	\details The function checks the data in the buffer and returns nonzero,
	when the buffer contain first 'buflen' bytes from a photometry file.
	This function is used for file format autodetection.
	\return nonzero if the file is a photometry file, zero otherwise
*/
int cmpack_pht_test_buffer(const char *buffer, int buflen, int filesize);

/******************* File header ***************************/

/**
	\brief Get frame size in pixels
	\param[in] file 		file context
	\param[out] width			frame width in pixels (0 = not available)
	\param[out] height			frame height in pixels (0 = not available)
	\return nonzero on success, zero on failure
*/
int cmpack_pht_get_size(CmpackPhtFile *file, int *width, int *height);

/**
	\brief Get julian date of observation
	\param[in] fc				file context
	\return Julian date or zero on failure
*/
double cmpack_pht_get_jd(CmpackPhtFile *fc);

/**
	\brief Get exposure duration
	\param[in] fc				file context
	\return Exposure duration in seconds or zero on failure
*/
double cmpack_pht_get_exptime(CmpackPhtFile *fc);

/**
	\brief Get optical filter name
	\details The function returns pointer to the internal buffer. 
	You must not free or modify it.
	\param[in] fc				file context
	\return Filter name, null terminated string
*/
const char *cmpack_pht_get_filter(CmpackPhtFile *fc);

/**
	\brief Get frame size in pixels
	\param[in] file 			file context
	\param[out] offsetx			frame offset in x axis in pixels
	\param[out] offsety			frame offset in y axis in pixels
	\return nonzero on success, zero on failure
*/
int cmpack_pht_get_offset(CmpackPhtFile *file, double *offsetx, double *offsety);

/**
	\brief Set parameter value in file header (string)
	\details If the parameter does not exist, it makes new entry 
	in the file header. Otherwise, its content is changed. 
	Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\param[in] com			comment text (can be NULL)
*/
void cmpack_pht_pkys(CmpackPhtFile *file, const char *key, const char *val, const char *com);

/**
	\brief Set parameter value in file header (integer)
	\details If the parameter does not exist, it makes new entry 
	in the file header. Otherwise, its content is changed. 
	Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\param[in] com			comment text (can be NULL)
*/
void cmpack_pht_pkyi(CmpackPhtFile *file, const char *key, int val, const char *com);

/**
	\brief Set parameter value in file header (double)
	\details If the parameter does not exist, it makes new entry 
	in the file header. Otherwise, its content is changed. 
	Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] val			parameter value
	\param[in] prec			number of decimal places
	\param[in] com			comment text (can be NULL)
*/
void cmpack_pht_pkyd(CmpackPhtFile *file, const char *key, double val, int prec, const char *com);

/**
	\brief Get value of parameter in file header (string)
	\details Keys are case sensitive. The function returns pointer to 
	the internal buffer. You must not free or modify it.
	\param[in] file			file context
	\param[in] key			parameter name
	\return parameter value, null terminated string
*/
const char *cmpack_pht_gkys(CmpackPhtFile *file, const char *key);

/**
	\brief Get value of parameter in file header (integer number)
	\details Keys are case sensitive. If the parameter is not defined,
	returns the default value.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] defval		default value
	\return parameter value or default value if it is not defined
*/
int cmpack_pht_gkyi(CmpackPhtFile *file, const char *key, int defval);

/**
	\brief Get value of parameter in file header (real number)
	\details Keys are case sensitive. If the parameter is not defined,
	returns the default value.
	\param[in] file			file context
	\param[in] key			parameter name
	\param[in] defval		default value
	\return parameter value or default value if it is not defined
*/
double cmpack_pht_gkyd(CmpackPhtFile *file, const char *key, double defval);

/**
	\brief Get comment text of parameter in file header
	\details Keys are case sensitive. The function returns pointer to 
	the internal buffer. You must not free or modify it.
	\param[in] file			file context
	\param[in] key			parameter name
	\return comment text, null terminated string
*/
const char *cmpack_pht_gkyc(CmpackPhtFile *file, const char *key);

/**
	\brief Get number of parameters in header section
	\param[in] file			file context
	\return number of parameter or zero if the file
	is not a valid context or if the file header is empty.
*/
int cmpack_pht_nkey(CmpackPhtFile *file);

/**
	\brief Get the parameter from header section by its index
	\details Index of the first parameter is zero. The output
	parameters receive the pointers to internal memory buffers.
	You must not free or modify it!
	\param[in] file			file context
	\param[in] index		parameter index (starting by zero)
	\param[out] key			parameter name
	\param[out] val			parameter value
	\param[out] com			comment text
	\return nonzero on success, zero on failure
*/
int cmpack_pht_gkyn(CmpackPhtFile *file, int index, const char **key, const char **val, const char **com);

/**
	\brief Delete parameter from header section
	\details Keys are case sensitive.
	\param[in] file			file context
	\param[in] key			parameter name
*/
void cmpack_pht_dkey(CmpackPhtFile *file, const char *key);

/************************* Table of apertures ********************************/

/**
	\brief Get number of apertures stored in a photometry file
	\param[in]	file 		file context
	\return number of apertures or negative value on failure
*/
int cmpack_pht_naper(CmpackPhtFile *file);

/**
	\brief Get aperture context by the index.
	\details Index of the first aperture is zero. The function 
	increments aperture's reference counter, the caller is 
	responsible to call the cmpack_unref() function	when 
	the context is no longer needed.
	\param[in] file			file context
	\param[in] index		aperture index (starting by 0)
	\return pointer to context or zero on failure.
*/
CmpackPhtAperture *cmpack_pht_get_aperture(CmpackPhtFile *file, int index);

/**
	\brief Get aperture context by the identifier.
	\details The function increments aperture's reference counter, 
	the caller is responsible to call the cmpack_unref() function 
	when the context is no longer needed.
	\param[in] file			file context
	\param[in] aperid		aperture identifider
	\return aperture index or negative value on failure
*/
CmpackPhtAperture *cmpack_pht_find_aperture(CmpackPhtFile *file, int aperid);

/** 
	\brief Add new aperture to a file
	\details The function steals the aperture's reference, so the caller
	dont need to decrement the aperture's reference counter.
	\param[in] file			file context
	\param[in] aper			aperture context
*/
void cmpack_pht_add_aperture(CmpackPhtFile *file, CmpackPhtAperture *aper);

/**
	\brief Create a new aperture definition
	\details The function makes a new aperture definition and
	returns a new reference to it.
	\param[in] id			aperture identifider
	\return pointer to the new reference
*/
CmpackPhtAperture *cmpack_pht_aperture_new(int id);

/** 
	\brief Get identifier of an aperture
	\param[in] aper			aperture context
	\return returns aperture id or zero on failure
*/
int cmpack_pht_aperture_get_id(CmpackPhtAperture *aper);

/**
	\brief Set aperture radius
	\param[in] aper			aperture context
	\param[in] radius		radius in pixels
	\return zero on success, error code on failure
*/
void cmpack_pht_aperture_set_radius(CmpackPhtAperture *aper, double radius);

/** 
	\brief Get radius of an aperture
	\param[in] aper			aperture context
	\return aperture radius in pixels or zero on failure
*/
double cmpack_pht_aperture_get_radius(CmpackPhtAperture *aper);

/*************************** Table of stars **********************************/

/**
	\brief Get number of stars stored in a photometry file
	\param[in]	file 		file context
	\return zero on success, error code on failure
*/
int cmpack_pht_nstar(CmpackPhtFile *file);

/**
	\brief Add new star to a file
	\details The function steals the reference to the star.
	\param[in] file			file context
	\param[in] star			star context
*/
void cmpack_pht_add_star(CmpackPhtFile *file, CmpackPhtStar *star);

/**
	\brief Get star context by the index.
	\details Index of the first star is zero. The function 
	increments star's reference counter, the caller is 
	responsible to call the cmpack_unref() function	when 
	the context is no longer needed.
	\param[in] file			file context
	\param[in] index		star index (starting by 0)
	\return pointer to context or zero on failure.
*/
CmpackPhtStar *cmpack_pht_get_star(CmpackPhtFile *file, int index);

/**
	\brief Get star context by its identifier.
	\details The function increments star's reference counter, 
	the caller is responsible to call the cmpack_unref() function 
	when the context is no longer needed.
	\param[in] file			file context
	\param[in] starid		star identifier
	\return pointer to context or zero on failure.
*/
CmpackPhtStar *cmpack_pht_find_star(CmpackPhtFile *file, int starid);

/**
	\brief Get star context by its reference identifier.
	\details The function increments star's reference counter, 
	the caller is responsible to call the cmpack_unref() function
	when the context is no longer needed.
	\param[in] file			file context
	\param[in] refid		reference identifier
	\return pointer to context or zero on failure.
*/
CmpackPhtStar *cmpack_pht_find_star_by_refid(CmpackPhtFile *file, int refid);

/**
	\brief Create a new star context
	\param[in] id			star identifier
	\return Zero on success or error code on failure
*/
CmpackPhtStar *cmpack_pht_star_new(int id);

/**
	\brief Get position of a center of a star
	\param[in] star			star context
	\return Star identifier or negative value on failure
*/
int cmpack_pht_star_get_id(CmpackPhtStar *star);

/**
	\brief Set position of a center of a star
	\param[in] star			star context
	\param[in] x			x position in pixels
	\param[in] y			y position in pixels
*/
void cmpack_pht_star_set_center(CmpackPhtStar *star, double x, double y);

/**
	\brief Get position of a center of a star
	\param[in] star			star context
	\param[out] x			x position in pixels
	\param[out] y			y position in pixels
	\return nonzero on success or zero on failure
*/
int cmpack_pht_star_get_center(CmpackPhtStar *star, double *x, double *y);

/**
	\brief Set reference id of a star
	\param[in] star			star context
	\param[in] refid		reference star identifier
*/
void cmpack_pht_star_set_refid(CmpackPhtStar *star, int refid);

/**
	\brief Get reference id of a star
	\param[in] star			star context
	\return Reference id or negative value on failure
*/
int cmpack_pht_star_get_refid(CmpackPhtStar *star);

/**
	\brief Clear all measurements for specified star
	\param[in] star			star context
*/
void cmpack_pht_star_clear_measurements(CmpackPhtStar *star);

/**
	\brief Set a measurement for specified star and aperture
	\param[in] star			star context
	\param[in] aperture		aperture identifier
	\param[in] data			measurement data
*/
void cmpack_pht_star_set_measurement(CmpackPhtStar *star, int aperture, 
		const CmpackPhtMeasurement *data);

/**
	\brief Set a measurement for specified star and aperture
	\param[in] star			star context
	\param[in] aperture		aperture identifier
	\param[out] data		measurement data
	\return Zero on success or error code on failure
*/
int cmpack_pht_star_get_measurement(CmpackPhtStar *star, int aperture, 
		CmpackPhtMeasurement *data);

#ifdef __cplusplus
}
#endif

#endif
